package com.example.accessiblespeech;

import android.content.Intent;
import android.content.SharedPreferences;
import android.media.MediaPlayer;
import android.os.Bundle;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;

import android.text.Html;
import android.text.TextUtils;
import android.util.Log;
import android.view.MenuItem;
import android.view.View;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;

import com.example.accessiblespeech.model.Category;
import com.example.accessiblespeech.model.MainPage;
import com.example.accessiblespeech.model.Recording;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.installations.FirebaseInstallations;

import java.io.IOException;
import java.lang.reflect.Array;
import java.util.ArrayList;

public class MainActivity extends AppCompatActivity {
    public static final String TEXT_TO_SEND = "com.example.accessiblespeech.TEXT_TO_SEND";
    private Button disability;
    private ListView listView;
    private BottomNavigationView bottomBar;
    private Button buttonPlus;
    private ListView list;
    private MediaPlayer mediaPlayer = null;
    private MediaPlayer player2 = null;
    private boolean isPlaying = false;
    private String disPath;
    private String installationID;
    private boolean updateCount2;
    private boolean flag2;

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ThemeUtils.onCreateSetTheme(this);
        setContentView(R.layout.activity_main);
        buttonPlus = findViewById(R.id.buttonPlus);
        disability = findViewById(R.id.disability);
        listView = findViewById(R.id.listView);
        bottomBar = findViewById(R.id.bottomBar3);
        bottomBar.setSelectedItemId(R.id.recordings_page);
        list = findViewById(R.id.frequentlyUsedList);
        disPath = getExternalCacheDir().getAbsolutePath() + "/Disability6000.3gp";
        updateCount2 = true;
        flag2 = true;

        ActionBar actionBar = getSupportActionBar();
        actionBar.setTitle(Html.fromHtml("<font color='#656565'>AccessibleSpeech</font>"));

        SharedPreferences sharedPreferences = getSharedPreferences("preferences", 0);
        installationID = sharedPreferences.getString("installationID", installationID);

        //https://firebase.google.com/docs/projects/manage-installations
        FirebaseInstallations.getInstance().getId()
                .addOnCompleteListener(new OnCompleteListener<String>() {
                    @Override
                    public void onComplete(@NonNull Task<String> task) {
                        if (task.isSuccessful()) {
                            sharedPreferences.edit().putString("installationID", task.getResult()).apply();
                            sharedPreferences.edit().putBoolean("device_installed", true);
                            System.out.println("Installation ID: " + task.getResult());
                        } else {
                            System.out.println("Failed getting installation ID");
                        }
                    }
                });

        if(!TextUtils.isEmpty(installationID)) {
            DatabaseReference deviceDB = FirebaseDatabase.getInstance().getReference("devices").child(installationID);
            deviceDB.addListenerForSingleValueEvent(new ValueEventListener() {
                @Override
                public void onDataChange(DataSnapshot snapshot) {
                    if (!snapshot.exists()) {
                        DatabaseReference db = FirebaseDatabase.getInstance().getReference("devices").child(installationID).child("categories");
                        String[] categoryNames = {"Health", "Shopping", "Social", "Food", "Questions", "Quick Replies", "Other"};
                        for (int i = 0; i < categoryNames.length; i++) {
                            String categoryId = db.push().getKey();
                            Category category = new Category(categoryNames[i]);
                            db.child(categoryId).setValue(category);
                        }
                    }
                }

                @Override
                public void onCancelled(DatabaseError error) {
                    System.out.println("creating device failed");
                }
            });
        }

        String[] categoriesKeys = new String[7];
        DatabaseReference d = FirebaseDatabase.getInstance().getReference("devices/"+installationID+"/categories");
        ValueEventListener categoryListener = new ValueEventListener() {
            @Override
            public void onDataChange(DataSnapshot dataSnapshot) {
                for (DataSnapshot childSnapshot: dataSnapshot.getChildren()) {
                    Category category = childSnapshot.getValue(Category.class);
                    /*
                    * 0: health
                    * 1: shopping
                    * 2: social
                    * 3: food
                    * 4: questions
                    * 5: quick replies
                    * 6: other
                     */
                    if(category.name.equals("Health"))
                        categoriesKeys[0] = childSnapshot.getKey();
                    else if(category.name.equals("Shopping"))
                        categoriesKeys[1] = childSnapshot.getKey();
                    else if(category.name.equals("Social"))
                        categoriesKeys[2] = childSnapshot.getKey();
                    else if(category.name.equals("Food"))
                        categoriesKeys[3] = childSnapshot.getKey();
                    else if(category.name.equals("Questions"))
                        categoriesKeys[4] = childSnapshot.getKey();
                    else if(category.name.equals("Quick Replies"))
                        categoriesKeys[5] = childSnapshot.getKey();
                    else if(category.name.equals("Other"))
                        categoriesKeys[6] = childSnapshot.getKey();
                }
                ((GlobalUtils) getApplication()).setCategoryKeys(categoriesKeys);
            }

            @Override
            public void onCancelled(DatabaseError databaseError) {
                System.out.println("getting categories failed");
            }
        };
        d.addValueEventListener(categoryListener);

        buttonPlus.setOnClickListener(new View.OnClickListener(){
            @Override
            public void onClick(View v){
                openRecordings();
            }
        });

        disability.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v){
                // Based on: https://developer.android.com/guide/topics/media/mediarecorder
                if (!isPlaying) {
                    mediaPlayer = new MediaPlayer();
                    mediaPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {
                        @Override
                        public void onCompletion(MediaPlayer mediaPlayer) {
                            mediaPlayer.release();
                            mediaPlayer = null;
                            isPlaying = false;
                        }
                    });
                    try {
                        mediaPlayer.setDataSource(disPath);
                        mediaPlayer.prepare();
                        mediaPlayer.start();
                    } catch (IOException e) {
                        System.out.println("failed to play recording");
                    }
                } else {
                    mediaPlayer.release();
                    mediaPlayer = null;
                }
                isPlaying = !isPlaying;
            }
        });

        final ArrayList<String> frequentlyUsedList = new ArrayList<>();
        ArrayList<Recording> list5 = new ArrayList<>();
        final ArrayAdapter adapterF = new ArrayAdapter(this, R.layout.list_item, frequentlyUsedList);
        list.setAdapter(adapterF);

        DatabaseReference referenceF = FirebaseDatabase.getInstance().getReference("devices/"+installationID+"/recordings");
        referenceF.addValueEventListener(new ValueEventListener() {
            @Override
            public void onDataChange(@NonNull DataSnapshot snapshot) {
                frequentlyUsedList.clear();
                for (DataSnapshot dataSnapshot : snapshot.getChildren()) {
                    Recording recording = dataSnapshot.getValue(Recording.class);
                    if (flag2){list5.add(recording);}
                }
                for (int i = 0; i < list5.size(); i++){
                    for (int j = i+1; j < list5.size(); j++) {
                        Recording p = list5.get(i);
                        Recording q = list5.get(j);
                        long currentNum = p.getCounter();
                        long nextNum = q.getCounter();
                        if (currentNum < nextNum) {
                            Recording temp = p;
                            list5.set(i,q);
                            list5.set(j, temp);
                        }
                    }
                }
                if((list5.size() != 0) && list5.size() > 4){
                    for (int z = 0; z<4; z++) {
                        Recording fin = list5.get(z);
                        frequentlyUsedList.add(fin.name);
                    }
                } else {
                    for (int y = 0;y < list5.size(); y++) {
                        Recording fin = list5.get(y);
                        frequentlyUsedList.add(fin.name);
                    }
                }
                adapterF.notifyDataSetChanged();
            }

            @Override
            public void onCancelled(@NonNull DatabaseError error) {
                Log.w("firebase", "loadPost:onCancelled", error.toException());
            }
        });

        list.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                String rec = (String) adapterF.getItem(position);
                DatabaseReference reference5 = FirebaseDatabase.getInstance().getReference("devices/"+installationID+"/recordings");
                reference5.addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot snapshot) {
                        for(DataSnapshot dataSnapshot:snapshot.getChildren()){
                            Recording recording2 = dataSnapshot.getValue(Recording.class);
                            if((rec).equals(recording2.name)){
                                DatabaseReference reference7 = FirebaseDatabase.getInstance().getReference("devices/"+installationID+"/recordings/"+dataSnapshot.getKey());
                                long count2 = recording2.getCounter()+1;
                                if (updateCount2) {
                                    reference7.child("counter").setValue(count2);
                                    updateCount2 = false;
                                    flag2=false;

                                    String path = recording2.getFilepath();
                                    player2 = new MediaPlayer();
                                    try {
                                        player2.setDataSource(path);
                                        player2.prepare();
                                        player2.start();
                                    } catch(IOException e) {
                                        Log.e("PlayTest", "failed");
                                    }
                                }
                            }
                        }
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError error) {
                        Log.w("firebase", "loadPost:onCancelled", error.toException());
                    }
                });
                updateCount2 = true;
            }
        });

        final ArrayList<MainPage> listc = new ArrayList<>();

        listc.add(new MainPage("Health",R.drawable.health));
        listc.add(new MainPage("Shopping",R.drawable.shopping));
        listc.add(new MainPage("Social",R.drawable.social));
        listc.add(new MainPage("Food",R.drawable.food));
        listc.add(new MainPage("Questions",R.drawable.questions));
        listc.add(new MainPage("Quick Replies",R.drawable.quick));
        listc.add(new MainPage("Other",R.drawable.others));

        Adapter adapter = new Adapter(MainActivity.this, R.id.gone, listc);

        listView.setAdapter(adapter);

        listView.setOnItemClickListener(new AdapterView.OnItemClickListener() {
            @Override
            public void onItemClick(AdapterView<?> parent, View view, int position, long id) {
                String item = (String) listc.get(position).name;
                Intent intent = new Intent(MainActivity.this, SpecificRecordingsActivity.class);
                intent.putExtra(TEXT_TO_SEND, item);
                startActivity(intent);
            }
        });

        bottomBar.setOnNavigationItemSelectedListener(new BottomNavigationView.OnNavigationItemSelectedListener() {
            @Override
            public boolean onNavigationItemSelected(@NonNull MenuItem item) {
                if(item.getItemId()== R.id.settings_page){
                    Intent intent = new Intent(MainActivity.this, SettingsActivity.class);
                    intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
                    startActivity(intent);
                } if(item.getItemId()== R.id.tts_page) {
                    Intent intent = new Intent(MainActivity.this, TTSActivity.class);
                    intent.addFlags(Intent.FLAG_ACTIVITY_REORDER_TO_FRONT);
                    startActivity(intent);
                }
                return false;
            }
        });
    }

    @Override
    public void onResume(){
        super.onResume();
        SharedPreferences sharedPreferences = getSharedPreferences("preferences", 0);

        if(((GlobalUtils) getApplication()).getChangeThemeMain()){
            ((GlobalUtils) getApplication()).setChangeThemeMain(false);
            recreate();
        }

        if(sharedPreferences.getBoolean("device_installed", true)){
            sharedPreferences.edit().putBoolean("device_installed", false).apply();
            recreate();
        }
    }

    public void openRecordings(){
        Intent intent = new Intent(this, RecordingActivity.class);
        startActivity(intent);
    }
}


