package com.example.accessiblespeech;

import androidx.annotation.NonNull;
import androidx.appcompat.app.ActionBar;
import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.core.app.ActivityCompat;
import androidx.core.widget.CompoundButtonCompat;

import android.Manifest;
import android.content.Intent;
import android.content.SharedPreferences;
import android.content.pm.PackageManager;
import android.graphics.Color;
import android.graphics.PorterDuff;
import android.graphics.drawable.Drawable;
import android.content.res.ColorStateList;
import android.media.MediaPlayer;
import android.media.MediaRecorder;
import android.os.Build;
import android.text.Html;
import android.text.TextUtils;
import android.util.Log;
import android.view.Gravity;
import android.view.LayoutInflater;
import android.view.MenuItem;
import android.view.View;
import android.view.ViewGroup;
import android.widget.Button;
import android.widget.CheckBox;
import android.widget.EditText;
import android.os.Bundle;
import android.widget.ImageButton;
import android.widget.LinearLayout;
import android.widget.PopupWindow;
import android.widget.Toast;

import com.example.accessiblespeech.model.Category;
import com.example.accessiblespeech.model.Recording;
import com.google.android.gms.tasks.OnCompleteListener;
import com.google.android.gms.tasks.Task;
import com.google.android.material.bottomnavigation.BottomNavigationView;
import com.google.firebase.database.DataSnapshot;
import com.google.firebase.database.DatabaseError;
import com.google.firebase.database.DatabaseReference;
import com.google.firebase.database.FirebaseDatabase;
import com.google.firebase.database.ValueEventListener;
import com.google.firebase.installations.FirebaseInstallations;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

public class RecordingActivity extends AppCompatActivity {
    private Button record;
    private Button play;
    private Button cancel;
    private Button save;
    private Button save1;
    private Button cancel1;
    private CheckBox health;
    private CheckBox shopping;
    private CheckBox social;
    private CheckBox food;
    private CheckBox questions;
    private CheckBox quick;
    private CheckBox other;
    private CheckBox disCheck;
    private MediaRecorder mediaRecorder = null;
    private MediaPlayer mediaPlayer = null;
    private EditText editName = null;
    private boolean isRecording = false;
    private boolean isPlaying = false;
    private boolean recordingExists = false;
    private String fileName = null;
    private String recordingName = null;
    private long counter;
    String[] categoriesKeys = new String[7];
    private PopupWindow popupWindow;
    private String disPath;
    private String installationID;
    private int currentTheme;
    private boolean flag;

    private static final int STEEL_BLUE = Color.parseColor("#B0C4DE");
    private static final int BLUE_SPONGE = Color.parseColor("#5D92B1");
    private static final int THISTLE = Color.parseColor("#D8BFD8");
    private static final int EGGPLANT = Color.parseColor("#816687");
    private static final int WAVECREST = Color.parseColor("#99CDC9");
    private static final int CADET_BLUE = Color.parseColor("#5F9EA0");
    private static final int TEXT = Color.parseColor("#656565");

    private DatabaseReference fb;
    private FirebaseDatabase firebaseInstance;
    private DatabaseReference firebaseDatabaseCategories;

    private String recordingId;
    private ConstraintLayout constraintLayout;

    // Code sample taken from: https://developer.android.com/guide/topics/media/mediarecorder
    private static final int REQUEST_RECORD_AUDIO_PERMISSION = 200;
    private boolean permissionToRecordAccepted = false;
    private String [] permissions = {Manifest.permission.RECORD_AUDIO};

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        switch (requestCode){
            case REQUEST_RECORD_AUDIO_PERMISSION:
                permissionToRecordAccepted  = grantResults[0] == PackageManager.PERMISSION_GRANTED;
                break;
        }
        if (!permissionToRecordAccepted ) {
            finish();
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        ThemeUtils.onCreateSetTheme(this);
        setContentView(R.layout.activity_recording);
        constraintLayout = findViewById(R.id.cl);
        record = findViewById(R.id.record);
        play = findViewById(R.id.play);
        cancel = findViewById(R.id.cancel);
        save = findViewById(R.id.save);
        disCheck = findViewById(R.id.disCheck);
        editName = findViewById(R.id.editName);
        disPath = getExternalCacheDir().getAbsolutePath() + "/Disability6000.3gp";
        isRecording = false;
        isPlaying = false;
        flag = false;

        ActionBar actionBar = getSupportActionBar();
        actionBar.setTitle(Html.fromHtml("<font color='#656565'>AccessibleSpeech</font>"));

        ActivityCompat.requestPermissions(this, permissions, REQUEST_RECORD_AUDIO_PERMISSION);

        SharedPreferences sharedPreferences = getSharedPreferences("preferences", 0);
        installationID = sharedPreferences.getString("installationID", installationID);
        currentTheme = sharedPreferences.getInt("theme", currentTheme);
        categoriesKeys = ((GlobalUtils) getApplication()).getCategoryKeys();

        setCheckBoxTheme(disCheck);

        firebaseInstance = FirebaseDatabase.getInstance();

        // get reference to 'recordings' node
        fb = firebaseInstance.getReference("devices/"+installationID+"/recordings");

        record.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                flag = true;
                String tempName = editName.getText().toString();
                fb.addValueEventListener(new ValueEventListener() {
                    @Override
                    public void onDataChange(@NonNull DataSnapshot snapshot) {
                        boolean alreadyExists = false;
                        if(flag) {
                            for (DataSnapshot dataSnapshot : snapshot.getChildren()) {
                                if ((dataSnapshot.child("name").getValue()).equals(tempName)) {
                                    Toast.makeText(getApplicationContext(), "Recording already exists", Toast.LENGTH_SHORT).show();
                                    alreadyExists = true;
                                }
                            }
                            if(!alreadyExists) {
                                recordingName = tempName;
                                // Record to the external cache directory for visibility
                                fileName = getExternalCacheDir().getAbsolutePath();
                                fileName += "/" + recordingName + ".3gp";

                                if (((recordingName.length() > 0) || disCheck.isChecked()) && (!isPlaying)) {
                                    // Based on: https://developer.android.com/guide/topics/media/mediarecorder
                                    if (!isRecording) {
                                        mediaRecorder = new MediaRecorder();
                                        mediaRecorder.setAudioSource(MediaRecorder.AudioSource.MIC);
                                        mediaRecorder.setOutputFormat(MediaRecorder.OutputFormat.THREE_GPP);
                                        mediaRecorder.setOutputFile(fileName);
                                        mediaRecorder.setAudioEncoder(MediaRecorder.AudioEncoder.AMR_NB);
                                        record.setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.ic_michollow, 0, 0, 0);
                                        try {
                                            mediaRecorder.prepare();
                                            mediaRecorder.start();
                                        } catch (IOException e) {
                                            System.out.println("failed to start recording");
                                        }
                                    } else {
                                        mediaRecorder.stop();
                                        mediaRecorder.release();
                                        mediaRecorder = null;
                                        recordingExists = true;
                                        record.setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.ic_mic, 0, 0, 0);
                                    }
                                    isRecording = !isRecording;
                                }
                            }
                        }
                        flag = false;
                    }

                    @Override
                    public void onCancelled(@NonNull DatabaseError error) {
                        System.out.println("getting recordings failed");
                    }
                });
            }
        });

        play.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if((((recordingName != null) && (recordingName.length() > 0)) || disCheck.isChecked()) && (!isRecording)){
                    // Based on: https://developer.android.com/guide/topics/media/mediarecorder
                    if (!isPlaying) {
                        play.setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.ic_baseline_stop_24,0,0,0);
                        mediaPlayer = new MediaPlayer();
                        mediaPlayer.setOnCompletionListener(new MediaPlayer.OnCompletionListener() {
                            @Override
                            public void onCompletion(MediaPlayer mediaPlayer) {
                                play.setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.ic_baseline_play_arrow_24,0,0,0);
                                mediaPlayer.release();
                                mediaPlayer = null;
                                isPlaying = false;
                            }
                        });
                        try {
                            mediaPlayer.setDataSource(fileName);
                            mediaPlayer.prepare();
                            mediaPlayer.start();
                        } catch (IOException e) {
                            System.out.println("failed to play recording");
                        }
                    } else {
                        play.setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.ic_baseline_play_arrow_24,0,0,0);
                        mediaPlayer.release();
                        mediaPlayer = null;
                    }
                    isPlaying = !isPlaying;
                }
            }
        });

        disCheck.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if(disCheck.isChecked()){
                    editName.setEnabled(false);
                    editName.setFocusable(false);
                } else {
                    editName.setEnabled(true);
                    editName.setFocusableInTouchMode(true);
                }
            }
        });

        cancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                Intent startMain = new Intent(getApplicationContext(), MainActivity.class);
                startActivity(startMain);
            }
        });

        save.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if((!isRecording) && (!isPlaying) && recordingExists) {
                    if (disCheck.isChecked()) {
                        File currentFile = new File(fileName);
                        File disFile = new File(disPath);
                        currentFile.renameTo(disFile);

                        Intent startMain = new Intent(getApplicationContext(), MainActivity.class);
                        startActivity(startMain);
                    }
                    else {
                        if(!recordingName.equals(editName.getText().toString())){
                            File currentFile = new File(fileName);
                            recordingName = editName.getText().toString();
                            fileName = getExternalCacheDir().getAbsolutePath();
                            fileName += "/"+recordingName+".3gp";
                            File newFile = new File(fileName);
                            currentFile.renameTo(newFile);
                        }
                        //popup window code based on: https://medium.com/@evanbishop/popupwindow-in-android-tutorial-6e5a18f49cc7
                        LayoutInflater inflater = (LayoutInflater) getApplicationContext().getSystemService(LAYOUT_INFLATER_SERVICE);
                        View popupView = inflater.inflate(R.layout.popup,null);
                        popupWindow = new PopupWindow(popupView, LinearLayout.LayoutParams.WRAP_CONTENT, LinearLayout.LayoutParams.WRAP_CONTENT);

                        health = (CheckBox) popupView.findViewById(R.id.health);
                        shopping = (CheckBox) popupView.findViewById(R.id.shopping);
                        social = (CheckBox) popupView.findViewById(R.id.social);
                        food = (CheckBox) popupView.findViewById(R.id.food);
                        questions = (CheckBox) popupView.findViewById(R.id.questions);
                        quick = (CheckBox) popupView.findViewById(R.id.quick);
                        other = (CheckBox) popupView.findViewById(R.id.other);
                        save1 = (Button) popupView.findViewById(R.id.save1);
                        cancel1 = (Button) popupView.findViewById(R.id.cancel1);

                        save1.setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.save,0,0,0);
                        cancel1.setCompoundDrawablesRelativeWithIntrinsicBounds(R.drawable.cancel,0,0,0);
                        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.M) {
                            save1.setCompoundDrawableTintList(ColorStateList.valueOf(TEXT));
                            cancel1.setCompoundDrawableTintList(ColorStateList.valueOf(TEXT));
                        }
                        if(currentTheme == 0) {
                            save1.getBackground().setColorFilter(STEEL_BLUE, PorterDuff.Mode.MULTIPLY);
                            cancel1.getBackground().setColorFilter(STEEL_BLUE, PorterDuff.Mode.MULTIPLY);
                        }
                        else if(currentTheme == 1) {
                            save1.getBackground().setColorFilter(THISTLE, PorterDuff.Mode.MULTIPLY);
                            cancel1.getBackground().setColorFilter(THISTLE, PorterDuff.Mode.MULTIPLY);
                        }
                        else if(currentTheme == 2) {
                            save1.getBackground().setColorFilter(WAVECREST, PorterDuff.Mode.MULTIPLY);
                            cancel1.getBackground().setColorFilter(WAVECREST, PorterDuff.Mode.MULTIPLY);
                        }

                        setCheckBoxTheme(health);
                        setCheckBoxTheme(shopping);
                        setCheckBoxTheme(social);
                        setCheckBoxTheme(food);
                        setCheckBoxTheme(questions);
                        setCheckBoxTheme(quick);
                        setCheckBoxTheme(other);

                        save1.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View view) {
                                ArrayList<String> keysList = new ArrayList<>();
                                if (health.isChecked())
                                    keysList.add(categoriesKeys[0]);
                                if (shopping.isChecked())
                                    keysList.add(categoriesKeys[1]);
                                if (social.isChecked())
                                    keysList.add(categoriesKeys[2]);
                                if (food.isChecked())
                                    keysList.add(categoriesKeys[3]);
                                if (questions.isChecked())
                                    keysList.add(categoriesKeys[4]);
                                if (quick.isChecked())
                                    keysList.add(categoriesKeys[5]);
                                if (other.isChecked())
                                    keysList.add(categoriesKeys[6]);
                                if (health.isChecked() || shopping.isChecked() || social.isChecked()
                                        || food.isChecked() || questions.isChecked() || quick.isChecked() || other.isChecked()) {

                                    createRecordingInDB(recordingName, fileName, counter, keysList);
                                    Intent startMain = new Intent(getApplicationContext(), MainActivity.class);
                                    startActivity(startMain);
                                }
                            }
                        });

                        cancel1.setOnClickListener(new View.OnClickListener() {
                            @Override
                            public void onClick(View view) {
                                popupWindow.dismiss();
                            }
                        });

                        popupWindow.showAtLocation(constraintLayout, Gravity.CENTER, 0, -25);
                    }
                }
            }
        });
    }

    private void createRecordingInDB(String name, String filepath, long counter, ArrayList<String> keysList){
        if(TextUtils.isEmpty(recordingId)){
            recordingId = fb.push().getKey();
        }

        Recording recording = new Recording(name, filepath, counter);
        recording.setCounter(0);
        fb.child(recordingId).setValue(recording);
        for(int i=0; i<keysList.size(); i++){
            DatabaseReference db = firebaseInstance.getReference("devices/"+installationID+"/categories/"+keysList.get(i)+"/recordings");
            db.child(recordingId).setValue(true);
        }

    }

    public void setCheckBoxTheme(CheckBox cb) {
        int checked = BLUE_SPONGE;
        int unchecked = BLUE_SPONGE;
        if(currentTheme == 0) {
            checked = BLUE_SPONGE;
            unchecked = BLUE_SPONGE;
        }
        else if(currentTheme == 1) {
            checked = EGGPLANT;
            unchecked = EGGPLANT;
        }
        else if(currentTheme == 2) {
            checked = CADET_BLUE;
            unchecked = CADET_BLUE;
        }
        int states[][] = {{android.R.attr.state_checked}, {}};
        int colors[] = {checked, unchecked};
        CompoundButtonCompat.setButtonTintList(cb, new ColorStateList(states, colors));
    }

    private void createCategory(String name){
        String categoryId = firebaseDatabaseCategories.push().getKey();

        Category category = new Category(name);
        firebaseDatabaseCategories.child(categoryId).setValue(category);
    }
}